<?php

namespace App\Http\Middleware;

use App\Modules\User\Services\QappUserService;
use Closure;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Exceptions\JWTException;
use Tymon\JWTAuth\Exceptions\TokenExpiredException;
use Tymon\JWTAuth\Middleware\BaseMiddleware;

class QuickAppGetUserFromToken extends BaseMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle(Request $request, Closure $next)
    {
        if (!$token = $this->auth->setRequest($request)->getToken()) {
            return response()->error('QAPP_NOT_LOGIN');
        }
        try {
            $user = $this->auth->authenticate($token);
            QappUserService::setGolableUserStatic($user->id);
        } catch (TokenExpiredException $e) {
            return response()->error('QAPP_TOKEN_ERROR');
        } catch (JWTException $e) {
            return response()->error('QAPP_TOKEN_ERROR');
        }
        if (!$user) {
            return response()->error('QAPP_NOT_LOGIN');
        }
        $this->events->fire('tymon.jwt.valid', $user);
        return $next($request);
    }
}
